import yaml
import click


def yaml_merge(source, destination):
    """
    Add source json objects to destination yaml
    """
    with open(source, "r") as s:
        source_yaml = yaml.safe_load(s)
    with open(destination, "r") as d:
        destination_yaml = yaml.safe_load(d)

    if not destination_yaml:
        click.echo(
            click.style(
                f"Unable to parse yaml file. Ensure that it is properly formatted and try again.",
                fg="red",
            )
        )
        raise Exception("ParseException: Unable to parse yaml file.")

    def item_gen(source, dest):
        if isinstance(source, dict):
            for key, val in source.items():
                if not dest or key not in dest:
                    dest[key] = val

                item_gen(val, dest[key])

    item_gen(source_yaml, destination_yaml)

    with open(destination, "w") as d:
        yaml.dump(destination_yaml, d, default_flow_style=False)



import json


def json_merge(source, destination):
    """
    Add source json objects to destination json
    """
    with open(source, "r") as s:
        source_json = json.load(s)
    with open(destination, "r") as d:
        destination_json = json.load(d)

    def item_gen(source, dest):
        if isinstance(source, dict):
            for key, val in source.items():
                if key not in dest:
                    dest[key] = val

                item_gen(val, dest[key])

    item_gen(source_json, destination_json)

    with open(destination, "w") as d:
        json.dump(destination_json, d, indent=2)



import click
import sys
from marlin_cli.api import archetypes, modules
import marlin_cli.constants as constants
import os


@click.command()
@click.option(
    "-t",
    "--type",
    "type",
    default="all",
    help="Type of entity to list, eityher module or archetype",
)
def list(type):
    """List all available modules or archetypes"""
    if type == "all":
        list_archetypes()
        list_modules()
    elif type == "archetype":
        list_archetypes()
    elif type == "module":
        list_modules()
    else:
        click.echo(
            f"Unknown type '{type}' available types are 'archetypes' and 'modules'"
        )
        sys.exit(1)


def list_archetypes():
    click.echo("Archetypes:")
    (archetype_list, error) = archetypes.list_archetypes()
    if error:
        click.echo(f"Error: {error}")
        sys.exit(1)
    for k in archetype_list:
        click.echo(f" - {k}")


def list_modules():
    click.echo("Modules:")
    (module_list, error) = modules.list_modules()
    if error:
        click.echo(f"Error: {error}")
        sys.exit(1)
    for k in module_list:
        click.echo(f" - {k}")



import click
from cfn_tools import load_yaml, dump_yaml


def cft_merge(source, destination):
    """
    Add source objects from a CFT to destination CFT
    """
    with open(source, "r") as s:
        raw = s.read()
        source_cft = load_yaml(raw)
    with open(destination, "r") as d:
        raw = d.read()
        destination_cft = load_yaml(raw)

    if not destination_cft:
        click.echo(
            click.style(
                f"Unable to parse yaml file. Ensure that it is properly formatted and try again.",
                fg="red",
            )
        )
        raise Exception("ParseException: Unable to parse yaml file.")

    def item_gen(source, dest):
        if isinstance(source, dict):
            for key, val in source.items():
                if not dest or key not in dest:
                    dest[key] = val

                item_gen(val, dest[key])

    item_gen(source_cft, destination_cft)

    with open(destination, "w") as d:
        d.write(dump_yaml(destination_cft))
